from flater.winforms.control import Control


class Button(Control):

    from System.Windows.Forms import Button

    _type = Button

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



from flater.basic import Basic


class Windows11Theme(Basic):

    from Telerik.WinControls.Themes import Windows11Theme

    _type = Windows11Theme
    _name = "Windows11"

    def __init__(self):
        super().__init__()


class Windows11DarkTheme(Basic):

    from Telerik.WinControls.Themes import Windows11DarkTheme

    _type = Windows11DarkTheme
    _name = "Windows11Dark"

    def __init__(self):
        super().__init__()



from flater.winforms.telerik.themes.theme_resolution_service import (
    ThemeResolutionService, SetDefaultTheme, GetDefaultTheme)

from flater.winforms.telerik.themes.windows11 import Windows11Theme, Windows11DarkTheme



class Basic(object):
    """
    基础类

    Attributes:
        _type (Class): 被初始化的类
    """

    _type = None

    def __init__(self):
        self._ = self._type()

    def Bind(self, Name: str, Func=None):
        """
        绑定事件

        Args:
            Name (str): 事件名称（请参见对应组件事件文档）
            Func (Callback): 事件函数（一般包括两个返回参数）

        Example:
            func1 = lambda e1, e2: print("Hello1")
            func2 = lambda e1=None, e2=None: print("Hello2")
            func3 = lambda e1, e2: print("Hello3")

            control.Bind("Click", func1)
            control.Bind("Click", func2)
            control.Bind("Click", func3)
            control.UnBind("Click", func2)

            # Hello1
            # Hello3
        """
        _ = getattr(self._, Name)
        _ += Func
        setattr(self._, Name, _)

    @property
    def Myself(self):
        return self._

    @Myself.setter
    def Myself(self, Class):
        self._ = Class

    def UnBind(self, Name: str, Func=None):
        """
        取消绑定事件

        Args:
            Name (str): 事件名称（请参见对应组件事件文档）
            Func (Callback): 事件函数
        """
        _ = getattr(self._, Name)
        _ -= Func
        setattr(self._, Name, _)



from flater.winforms.control import Control


class RadControl(Control):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def ThemeName(self):
        return self._.ThemeName

    @ThemeName.setter
    def ThemeName(self, Name: str):
        self._.ThemeName = Name



from flater.winforms.metro.metro_control import MetroControl
from flater.winforms.button import Button


class MetroButton(MetroControl, Button):

    from MetroFramework.Controls import MetroButton

    _type = MetroButton

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


from flater.winforms.control import Control


class MetroControl(Control):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def GetStyle(self):
        return self.Style

    def GetTheme(self):
        return self.Theme

    def SetStyle(self, Color: str):
        self.Style = Color

    def SetTheme(self, Style: str):
        self.Theme = Style

    @property
    def Style(self):
        """
        颜色样式
        """
        return self._.Style.ToString()

    @Style.setter
    def Style(self, Color: str):
        """
        颜色样式

        Args:
            Color (str): 颜色样式可为：Black，Blue，Brown，Default，Green，Lime，Magenta，Orange，Pink，Purple，Red，Silver，Teal，White，Yellow
        """

        from MetroFramework import MetroColorStyle

        self._.Style = getattr(MetroColorStyle, Color)

    @property
    def Theme(self):
        from MetroFramework import MetroThemeStyle

        return self._.Theme.ToString()

    @Theme.setter
    def Theme(self, Style: str):
        from MetroFramework import MetroThemeStyle, MetroColorStyle

        if Style == "System":
            from darkdetect import isDark
            if isDark():
                self._.Theme = getattr(MetroThemeStyle, "Dark")
            else:
                self._.Theme = getattr(MetroThemeStyle, "Light")
            return

        self._.Theme = getattr(MetroThemeStyle, Style)


from flater.basic import Basic


class MetroStyleManager(Basic):

    from MetroFramework.Components import MetroStyleManager

    _type = MetroStyleManager

    def __init__(self):
        super().__init__()

    def GetOwner(self):
        return self.Owner

    def GetStyle(self):
        return self.Style

    def GetTheme(self):
        return self.Theme

    @property
    def Owner(self):
        return self._.Owner

    @Owner.setter
    def Owner(self, Window):
        self._.Owner = Window._

    def SetOwner(self, Window):
        self.Owner = Window

    def SetStyle(self, Color: str):
        self.Style = Color

    def SetTheme(self, Style: str):
        self.Theme = Style

    @property
    def Style(self):
        """
        颜色样式
        """
        return self._.Style.ToString()

    @Style.setter
    def Style(self, Color: str):
        """
        颜色样式

        Args:
            Color (str): 颜色样式可为：Black，Blue，Brown，Default，Green，Lime，Magenta，Orange，Pink，Purple，Red，Silver，Teal，White，Yellow
        """

        from MetroFramework import MetroColorStyle

        self._.Style = getattr(MetroColorStyle, Color.capitalize())

    @property
    def Theme(self):
        return self._.Theme.ToString()

    @Theme.setter
    def Theme(self, Style: str):
        """
        主题

        Args:
            Color (str): 颜色样式可为：Black，Blue，Brown，Default，Green，Lime，Magenta，Orange，Pink，Purple，Red，Silver，Teal，White，Yellow
        """
        from MetroFramework import MetroThemeStyle

        if Style == "System":
            from darkdetect import isDark
            if isDark():
                self._.Theme = getattr(MetroThemeStyle, "Dark")
            else:
                self._.Theme = getattr(MetroThemeStyle, "Light")
            return

        self._.Theme = getattr(MetroThemeStyle, Style)



from flater.winforms.application import Application
from flater.winforms.button import Button
from flater.winforms.control import Control
from flater.winforms.form import Form

from System.Windows.Forms import AnchorStyles


N = AnchorStyles.Top
S = AnchorStyles.Bottom
W = AnchorStyles.Left
E = AnchorStyles.Right



from flater.library import libs1, libs2, libs3

# ---

from clr import AddReference

AddReference("System.Drawing")
AddReference("System.Windows")
AddReference("System.Windows.Forms")
AddReference("System.Runtime.InteropServices")


from System.Runtime.InteropServices import RuntimeInformation


# print("运行框架为", RuntimeInformation.FrameworkDescription)

def LoadTelerik():
    for lib in libs1:
        AddReference(libs1[lib])


def LoadMetro():
    for lib in libs2:
        AddReference(libs2[lib])


def LoadSkinSharp():
    for lib in libs3:
        AddReference(libs3[lib])

# ---

from flater.winforms import *



from flater.winforms.telerik.simple.widget import Widget


class Button(Widget):
    from flater.winforms.telerik.rad_button import RadButton

    _type = RadButton

    def __init__(self, *args, text: str = "", **kwargs):
        super().__init__(*args, **kwargs)

        self.configure(text=text)

    def configure(self, **kwargs):
        if "text" in kwargs:
            self._.Text = kwargs.pop("text")



from flater.winforms.telerik.simple.widget import Widget


class MainWindow(Widget):

    from flater.winforms.telerik.rad_form import RadForm

    _type = RadForm

    def __init__(self, master=None):
        super().__init__(master)
        from flater.winforms.application import Application
        from flater.winforms.telerik.themes.theme_resolution_service import ThemeResolutionService
        self.theme = None
        self.app = Application()
        self.theme_control = ThemeResolutionService()

    def add_to_titlebar_buttons(self, button: Widget):
        self._.AddToTitleBarButtons(button)

    def insert_to_titlebar_buttons(self, index: int, button: Widget):
        self._.InsertToTitleBarButtons(index, button)

    def create_control_spy_window(self):
        from flater.winforms.telerik.rad_control_spy_form import RadControlSpyForm
        self._control_spy = RadControlSpyForm()
        self._control_spy.Show()
        return self._control_spy

    def mainloop(self):
        self.app.Run(self._)

    def quit(self):
        self.app.Close(self._)

    def theme_names(self):
        return ["win11", "win11dark"]

    def theme_use(self, theme_name):
        if theme_name == "win11":
            from flater.winforms.telerik.themes.windows11 import Windows11Theme
            self.theme = Windows11Theme()
        elif theme_name == "win11dark":
            from flater.winforms.telerik.themes.windows11 import Windows11DarkTheme
            self.theme = Windows11DarkTheme()
        self.theme_control.ApplicationThemeName = self.theme

    def title(self, text: str = None):
        if text:
            self._.Text = text
        else:
            return self._.Text




import os

dir = os.path.abspath(os.path.dirname(__file__))

dir_telerik = os.path.join(dir, "telerik")
dir_telerik_net40 = os.path.join(dir_telerik, "net40")
dir_telerik_netcore = os.path.join(dir_telerik, "netcore")

dir_metro = os.path.join(dir, "metro")
dir_metro_net40 = os.path.join(dir_metro, "net40")

dir_skinsharp = os.path.join(dir, "skinsharp")
dir_skinsharp_2005_2008 = os.path.join(dir_metro, "Bin-2005-2008")

if "WINFORMS_RUNTIME" == os.environ:
    if os.environ["WINFORMS_RUNTIME"].lower() == "netcore":
        dir_telerik_default = dir_telerik_netcore
    else:
        dir_telerik_default = dir_telerik_net40
else:
    dir_telerik_default = dir_telerik_net40

# ---

libs1 = {}

if os.path.exists(dir_telerik_default):
    for lib in os.listdir(dir_telerik_default):
        path = os.path.join(dir_telerik_default, lib)
        if os.path.isfile(path):
            if os.path.splitext(path)[1] == ".dll":  # 判断文件扩展名是否为“.dll”
                libs1[lib] = path

libs2 = {}

if os.path.exists(dir_metro_net40):
    for lib in os.listdir(dir_metro_net40):
        path = os.path.join(dir_metro_net40, lib)
        if os.path.isfile(path):
            if os.path.splitext(path)[1] == ".dll":  # 判断文件扩展名是否为“.dll”
                libs2[lib] = path

libs3 = {}

if os.path.exists(dir_skinsharp_2005_2008):
    for lib in os.listdir(dir_skinsharp_2005_2008):
        path = os.path.join(dir_skinsharp_2005_2008, lib)
        if os.path.isfile(path):
            if os.path.splitext(path)[1] == ".dll":  # 判断文件扩展名是否为“.dll”
                libs3[lib] = path

"""
libs = {
    "RadControlSpy.dll":
        os.path.join(path, "RadControlSpy.dll"),
    "Telerik.WinControls.dll":
        os.path.join(path, "Telerik.WinControls.dll"),
    "Telerik.WinControls.Themes.Windows11.dll":
        os.path.join(path, "Telerik.WinControls.Themes.Windows11.dll"),
    "Telerik.WinControls.UI.dll":
        os.path.join(path, "Telerik.WinControls.UI.dll"),
}
"""



from flater.basic import Basic


class Application(Basic):
    """
    主程序，用于实现窗口的消息循环

    """

    from System.Windows.Forms import Application

    _type = Application

    def __init__(self):
        super().__init__()

    def Run(self, Window, _WinForms_Window=None):
        """
        运行窗口

        Args:
            Window (flater.Window): 窗口类
            _WinForms_Window (System.Windows.Forms.Application): 窗口类（应为WinForms的窗口）
        """
        if _WinForms_Window:
            self._.Run(_WinForms_Window)
        else:
            self._.Run(Window._)

    def Close(self, Window, _WinForms_Window=None):
        """
        关闭窗口

        Args:
            Window (flater.Window): 窗口类
            _WinForms_Window (System.Windows.Forms.Application): 窗口类（应为WinForms的窗口）
        """
        if _WinForms_Window:
            self._.Close(_WinForms_Window)
        else:
            self._.Close(Window._)


if __name__ == '__main__':
    app = Application()


from flater import LoadTelerik

LoadTelerik()

from flater.winforms.telerik.rad_button import RadButton
from flater.winforms.telerik.rad_control import RadControl
from flater.winforms.telerik.rad_control_spy_form import RadControlSpyForm
from flater.winforms.telerik.rad_form import RadForm

from flater.winforms.telerik.themes import *



from flater.winforms.metro import *
from flater import *

window = MetroForm()
window.StyleManager.Style = "Red"

button3 = MetroButton(Parent=window)
button3.Text = "System"
button3.Bind("Click", lambda e1, e2: window.StyleManager.SetTheme("System"))
button3.Pack(Dock="Top")

button2 = MetroButton(Parent=window)
button2.Text = "Dark"
button2.Bind("Click", lambda e1, e2: window.StyleManager.SetTheme("Dark"))
button2.Pack(Dock="Top")

button = MetroButton(Parent=window)
button.Text = "Light"
button.Bind("Click", lambda e1, e2: window.StyleManager.SetTheme("Light"))
button.Pack(Dock="Top", Margin=10)

window.AppRun()


from flater.winforms.telerik.rad_form import RadForm


class RadControlSpyForm(RadForm):

    from Telerik.WinControls.RadControlSpy import RadControlSpyForm

    _type = RadControlSpyForm

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



from flater.winforms.form import Form

from flater.winforms.metro.metro_control import MetroControl


class MetroForm(Form, MetroControl):
    from MetroFramework.Forms import MetroForm

    _type = MetroForm

    from enum import Enum

    class WindowButtons(Enum):
        Close = 0,
        Maximize = 1,
        Minimize = 2,

    def __init__(self, StyleManager: bool = True):
        super().__init__()

        from flater.winforms.metro.metro_style_manager import MetroStyleManager

        if StyleManager:
            self.StyleManager = MetroStyleManager()  # 全局设置主题
            self.StyleManager.Owner = self
            self.StyleManager.Theme = "Dark"
            self.StyleManager.Style = "Blue"

    @property
    def Movable(self):
        """
        是否可以移动

        Returns:
            bool: 返回布尔值

        """
        return self._.Movable

    @Movable.setter
    def Movable(self, Enable: bool):
        """
        是否可以移动

        Args:
            Enable (bool): 是否启用

        Returns:
            bool: 不返回

        """
        self._.Movable = Enable

    def RemoveCloseButton(self):
        self._.RemoveCloseButton()

    from flater.winforms.metro.metro_style_manager import MetroStyleManager



import clr

try:
    clr.AddReference("wpf\PresentationFramework")
except SyntaxWarning:
    pass
from System.IO import *
from System.Windows.Markup import XamlReader
from System.Windows import *
from System.Threading import Thread, ThreadStart, ApartmentState
from System.Windows.Controls import *


class MyWindow(Window):
    def __init__(self):
        try:
            stream = StreamReader('gui.xaml')
            self.window = XamlReader.Load(stream.BaseStream)
            ButtoninXAML = LogicalTreeHelper.FindLogicalNode(self.window, "button1")
            ButtoninXAML.Click += RoutedEventHandler(self.Button_Click)
            Application().Run(self.window)
        except Exception as ex:
            print(ex)

    def Button_Click(self, sender, e):
        print('clicked')


if __name__ == '__main__':
    thread = Thread(ThreadStart(MyWindow))
    thread.SetApartmentState(ApartmentState.STA)
    thread.Start()
    thread.Join()



from flater import LoadMetro

LoadMetro()

from flater.winforms.metro.metro_button import MetroButton
from flater.winforms.metro.metro_control import MetroControl
from flater.winforms.metro.metro_form import MetroForm
from flater.winforms.metro.metro_style_manager import MetroStyleManager




from flater.basic import Basic


def SetDefaultTheme(_ThemeName_Or_Theme):
    ThemeResolutionService().ApplicationThemeName = _ThemeName_Or_Theme


def GetDefaultTheme():
    return ThemeResolutionService().ApplicationThemeName


class ThemeResolutionService(Basic):
    from Telerik.WinControls import ThemeResolutionService

    _type = ThemeResolutionService

    def __init__(self):
        super().__init__()

    @property
    def ApplicationThemeName(self):
        return self._.ApplicationThemeName

    @ApplicationThemeName.setter
    def ApplicationThemeName(self, _ThemeName_Or_Theme):
        if isinstance(_ThemeName_Or_Theme, str):
            self._.ApplicationThemeName = _ThemeName_Or_Theme
        else:
            self._.ApplicationThemeName = _ThemeName_Or_Theme._name

    @property
    def SystemThemeName(self):
        return self._.SystemThemeName



from flater.winforms.telerik.rad_control import RadControl
from flater.winforms.button import Button


class RadButton(RadControl, Button):

    from Telerik.WinControls.UI import RadButton

    _type = RadButton

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


from typing import Callable


class Widget(object):

    _type = None

    def __init__(self, master=None):
        self._ = self._type()
        self.master = master
        if master:
            self.master._.Add(self._)

    def on_click(self, func: Callable):
        self._.Bind("Click", func)

    def place(self, x=None, y=None, width=None, height=None):
        self.pos(x, y)
        self.size(width, height)

    def pos(self, x=None, y=None):
        if x is None and y is None:
            return self._.Pos
        else:
            if x is None:
                x = self._.Pos[0]
            elif y is None:
                y = self._.Pos[1]
            self._.Pos = (x, y)
            print(x, y)

    def size(self, width=None, height=None):
        if width is None and height is None:
            return self._.Size
        else:
            if width is None:
                width = self._.Size[0]
            elif height is None:
                height = self._.Size[1]
        self._.Size = (width, height)



from System.Threading import Thread, ThreadStart, ApartmentState


def Run(Window):
    thread = Thread(ThreadStart(Window))
    thread.SetApartmentState(ApartmentState.STA)
    thread.Start()
    thread.Join()



from flater.basic import Basic


class Control(Basic):
    """
    基础组件，不可见
    """
    def __init__(self, Parent: Basic = None):
        """
        初始化组件

        Args:
            Parent (Control): 父组件，如果设置此类将自动调用 Parent.Add 添加该组件
        """
        super().__init__()

        if Parent:
            Parent.Add(self)

    def Add(self, Widget, _WinForms_Widget=None):
        """
        在容器组件中添加组件

        Args:
            Widget (flater.Window): 组件类
            _WinForms_Widget (System.Windows.Forms.Control): 组件类（应为WinForms的组件）
        """
        if _WinForms_Widget:
            self._.Controls.Add(_WinForms_Widget)
        else:
            self._.Controls.Add(Widget._)

    @property
    def Anchor(self):
        return self._.Anchor

    @Anchor.setter
    def Anchor(self, Style: str):
        self._.Anchor = Style

    @property
    def Dock(self):
        return self._.Dock

    @Dock.setter
    def Dock(self, Style):
        from System.Windows.Forms import DockStyle

        self._.Dock = (getattr(DockStyle, Style))

    @property
    def Enable(self):
        return self._.Enable

    @Enable.setter
    def Enable(self, Enable: bool):
        self._.Enable = Enable

    @property
    def Handle(self):
        return self._.Handle

    def Hide(self):
        self._.Hide()

    @property
    def Margin(self):
        return self._.Margin

    @Margin.setter
    def Margin(self, *args):
        from System.Windows.Forms import Padding

        self._.Margin = Padding(*args)

    def Margin2(self, *args):
        from System.Windows.Forms import Padding

        self._.Margin = Padding(*args)

    def Pack(self, Dock=None, Margin=None, Padding=None, Anchor=None):
        if Dock:
            self.Dock = Dock
        if Margin:
            if type(Margin).__name__ == "int":
                self.Margin = Margin
            elif type(Margin).__name__ == "tuple" or type(Margin).__name__ == "list":
                self.Margin2(*Margin),
        if Padding:
            if type(Padding).__name__ == "int":
                self.Margin = Padding
            elif type(Padding).__name__ == "tuple" or type(Padding).__name__ == "list":
                self.Margin2(*Padding)
        if Anchor:
            self.Anchor = Anchor

    @property
    def Padding(self):
        return self._.Padding

    @Margin.setter
    def Padding(self, *args):
        from System.Windows.Forms import Padding

        self._.Padding = Padding(*args)

    def Padding2(self, *args):
        from System.Windows.Forms import Padding

        self._.Padding = Padding(*args)

    def Place(self, X=None, Y=None, Width=None, Height=None):
        if X is None:
            X = self.Pos[0]
        if Y is None:
            Y = self.Pos[1]
        self.Pos = [X, Y]

        if Width is None:
            Width = self.Size[0]
        if Height is None:
            Height = self.Size[1]
        self.Size = [Width, Height]

    @property
    def Pos(self):
        return self._.Location.X, self._.Location.Y

    @Pos.setter
    def Pos(self, Tuple: tuple):
        from System.Drawing import Point
        self._.Location = Point(Tuple[0], Tuple[1])

    def Show(self):
        """
        显示窗口
        """
        self._.Show()

    @property
    def Size(self):
        return self._.Size.Width, self._.Size.Height

    @Size.setter
    def Size(self, Tuple: tuple):
        from System.Drawing import Size
        self._.Size = Size(Tuple[0], Tuple[1])

    @property
    def Text(self):
        return self._.Text

    @Text.setter
    def Text(self, String: str):
        self._.Text = String




from flater.basic import Basic


class Form(Basic):

    from System.Windows.Forms import Form

    _type = Form

    def __init__(self):
        super().__init__()

        self._app = None

    def Close(self):
        self._.Close()

    def AppRun(self):
        from flater.winforms.application import Application
        self._app = Application()
        self._app.Run(self)

    def AppClose(self):
        if self._app:
            self._app.Close()



from flater.winforms.telerik.simple import *

root = MainWindow()
root.title("Hello World")
root.theme_use("win11dark")

root.size(500, 600)

button = Button(root, text="Light")
button.on_click(lambda sender, args: root.theme_use("win11"))
button.place(0, 0, 150, 40)

button2 = Button(root, text="Dark")
button2.on_click(lambda sender, args: root.theme_use("win11dark"))
button2.place(0, 50, 150, 40)

# root.create_control_spy_window()
root.mainloop()



from flater.winforms.form import Form
from flater.winforms.telerik.rad_control import RadControl


class RadForm(RadControl, Form):

    from Telerik.WinControls.UI import RadForm

    _type = RadForm

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
